/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : harris_section.h
* Author             : AST Robotics group 
* Date First Issued  : 17 August 2009 : Version 1.0
* Description        : header for harris_section.c.
********************************************************************************
* History:
* 17 August 2009 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __HARRIS_SECTION_H__
#define __HARRIS_SECTION_H__

/* Includes ------------------------------------------------------------------*/
//#include "utils.h"
//#include "91x_lib.h"
#include "TMatrix.h"

/* Exported types ------------------------------------------------------------*/
typedef struct {
	u8 X, Y;
    u16 Score; // Harris Measure
} TCorner;

/* Exported constants --------------------------------------------------------*/

#define GlobalMaxHPt 99

#define MaxImgCol  180 // 60 80 160 180
#define MaxImgRow  135 // 45 60 120 135

#define Gauss1_0_0 0
#define Gauss1_0_1 1
#define Gauss1_0_2 0
#define Gauss1_1_0 1
#define Gauss1_1_1 3
#define Gauss1_1_2 1
#define Gauss1_2_0 0
#define Gauss1_2_1 1
#define Gauss1_2_2 0

#define Dx_0_0 -1
#define Dx_0_1 0
#define Dx_0_2 1
#define Dx_1_0 -1
#define Dx_1_1 0
#define Dx_1_2 1
#define Dx_2_0 -1
#define Dx_2_1 0
#define Dx_2_2 1

#define Dy_0_0 -1
#define Dy_0_1 -1
#define Dy_0_2 -1
#define Dy_1_0 0
#define Dy_1_1 0
#define Dy_1_2 0
#define Dy_2_0 1
#define Dy_2_1 1
#define Dy_2_2 1

#define Gauss2_0_0 10
#define Gauss2_0_1 16
#define Gauss2_0_2 19
#define Gauss2_0_3 16
#define Gauss2_0_4 10
#define Gauss2_1_0 16
#define Gauss2_1_1 26
#define Gauss2_1_2 31
#define Gauss2_1_3 26
#define Gauss2_1_4 16
#define Gauss2_2_0 19
#define Gauss2_2_1 31
#define Gauss2_2_2 36
#define Gauss2_2_3 31
#define Gauss2_2_4 19
#define Gauss2_3_0 16
#define Gauss2_3_1 26
#define Gauss2_3_2 31
#define Gauss2_3_3 26
#define Gauss2_3_4 16
#define Gauss2_4_0 10
#define Gauss2_4_1 16
#define Gauss2_4_2 19
#define Gauss2_4_3 16
#define Gauss2_4_4 10

#define d_G1    3
#define d_Der   3
#define d_G2    5
#define d_LM    5

#define ProcessRow (d_LM+d_G2+d_Der+d_G1-3) // 13
#define SCol(c) (c-d_G1+1)                // 78
#define SRow (ProcessRow-d_G1+1)            // 11
#define DerCol(c) (SCol(c)-d_Der+1)               // 76
#define DerRow (SRow-d_Der+1)               // 9
#define S2Col(c) (DerCol(c)-d_G2+1)               // 72
#define S2Row (DerRow-d_G2+1)               // 5
#define LMCol(c) (S2Col(c)-d_LM+1)                // 68
#define LMRow (S2Row)                       // 5

/* Exported variables --------------------------------------------------------*/
//extern THarrisPoint HarrisPoint[GlobalMaxHPt];
//extern u8 HPtCount;
//extern s32 kh1_inv;
//extern s32 HarrisTh;
//extern s32 MaxHPt;
//extern s32 ImgRow;
//extern s32 ImgCol;

extern TCorner *pFeature;
extern u8 HPtCount;
extern s32 kh1_inv, HarrisTh, MaxHPt;
extern void *MImgRows[ProcessRow];
extern void *MSImgRows[SRow];
extern void *MIx2Rows[DerRow];
extern void *MIy2Rows[DerRow];
extern void *MIxyRows[DerRow];
extern void *MsIx2Rows[S2Row];
extern void *MsIy2Rows[S2Row];
extern void *MsIxyRows[S2Row];
extern void *MHMRows[S2Row];
extern void *MLM1Rows[LMRow];
extern void *MLM2Rows[LMRow];

extern TMatrixU8 MImg;
extern TMatrixS32 MSImg;
extern TMatrixS32 MIx2;
extern TMatrixS32 MIy2;
extern TMatrixS32 MIxy;
extern TMatrixS32 MsIx2;
extern TMatrixS32 MsIy2;
extern TMatrixS32 MsIxy;
extern TMatrixS32 MHM;
extern TMatrixS32 MLM1;
extern TMatrixS32 MLM2;
/* Module private variables --------------------------------------------------*/
/* Exported macro ------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

void HarrisInit(int ImgCol);
//void Harris(u8* pImg);//, u32 ImgCol_V, u32 ImgRow_V);
void Harris(u8* pImg, u32 ImgCol, u32 ImgRow);

#endif /* __HARRIS_SECTION_H__ */

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
