/*******************************************************************************
 *                            C STMicroelectronics
 *    Reproduction and Communication of this document is strictly prohibited
 *      unless specifically authorized in writing by STMicroelectronics.
 *------------------------------------------------------------------------------
 *                              Imaging Division
 *------------------------------------------------------------------------------
 * $Id: c4mMath.h.rca 1.2 Wed Jul 23 10:36:10 2008 vitor.schwambach@st.com Experimental $
 *------------------------------------------------------------------------------
 * $Log: c4mMath.h.rca $
 * 
 *  Revision: 1.2 Wed Jul 23 10:36:10 2008 vitor.schwambach@st.com
 *  No modification.
 * 
 *  Revision: 1.1 Fri Jul 18 13:56:55 2008 vitor.schwambach@st.com
 *  Initial version from AST
 ******************************************************************************/

#ifndef __C4MMATH_H_
#define __C4MMATH_H_

#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif

#define C4M_SWAP(a, b, tmp)		{ tmp = (a); (a) = (b); (b) = tmp; }

#define C4M_TINY				1.0e-30
#define C4M_BIG					1.0e+30

#define C4M_PI					3.141592653589793
#define C4M_2PI					6.28318530717959
#define C4M_LOG_ONE_ON_SQRT2PI	-0.91893853320467

#define C4M_MIN(a, b)			((a) < (b) ? (a) : (b))
#define C4M_MAX(a, b)			((a) > (b) ? (a) : (b))

#define C4M_ABS(x)				((x) < 0 ? -(x) : (x))
#define C4M_FABS(x)				(fabs(x))

#define C4M_SQRT(x)				(sqrt(x))

#define C4M_ADD(a, b)			((a) + (b))

#define C4M_ROUND(x)			((x) >= 0 ? (int32)((x) + 0.5) : (int32)((x) - 0.5))
#define C4M_SIGN(a, b)			((b) >= 0.0 ? fabs(a) : -fabs(a))

#define C4M_SQR(a)				((a) * (a))
#define C4M_SQR_2(a)			((a) == 0.0 ? 0.0 : (a) * (a))

#define C4M_POW(a, b)			pow(a, b)

#define C4M_EXP10(a)			pow(10, a)

#define C4M_COS(a)				(cos(a))
#define C4M_SIN(a)				(sin(a))

#define C4M_LOG(a)				(log(a))
#define C4M_EXP(a)				(exp(a))

#define C4M_COS_Q(theta)		(-3.3574888e-01 * (theta) * (theta) - 1.0922668e-01 * (theta) + 1)
#define C4M_FREQ2MEL(f)			(2595.0 * log10(1.0 + (f) / 700.0))
#define C4M_MEL2FREQ(m)			((C4M_EXP10((m) / 2595.0) - 1.0) * 700.0)
/*#define C4M_MEL2FREQ(m)		((exp10((m) / 2595.0) - 1.0) * 700.0)*/

/* C4M_05_FIX: Calcola il valore 0.5 per un fixed point con F bits dopo la virgola */
#define C4M_05_FIX(f)			((f >= 0) ? (1 << ((f) - 1)) : 0)

/* C4M_ROUND_SHIFT_FIX: Calcola il valore shiftato e arrotondato */
#define C4M_ROUND_SHIFT_FIX(x, rm, f)	(((f) > 0) ? (((x) + (rm)) >> (f)) : ((x) << (-(f))))
/*#define	C4M_ROUND_SHIFT_FIX(X,F) (((F)>0) ? ((((X)>>((F)-1))+1)>>1) : ((X)<<(-(F)))) */


#ifdef __cplusplus
}
#endif

#endif /* __C4MMATH_H_ */
