/******************** (C) COPYRIGHT 2008 STMicroelectronics ********************
* File Name          : fifo.h
* Author             : AST-Robotics
* Date First Issued  : Feb 2008
* Description        : header of fifo.c
********************************************************************************
* History:
* 28 Feb 2008: v1.0
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

#ifndef _CHAR_FIFO_H_
#define _CHAR_FIFO_H_

//#define CHAR_FIFO_INLINE

/*Structures typedef----------------------------------------------------------*/
typedef struct { // Receiving and transmitting Pointerss
  u32 Max;
  u32 Full;
  u32 First;
  u32 Last;
  u8 *Buffer;
} TCharFifo;

/*Constants-------------------------------------------------------------------*/

/*----------------------------functions----------------------------------------*/

#ifndef CHAR_FIFO_INLINE

// Initialize a Fifo
void CharFifo_Init(TCharFifo *Fifo, u32 MaxElement, u8* DataBuffer);

// Clear the content of a Fifo
void CharFifo_Clear(TCharFifo *Fifo);

// Insert a new element in the Fifo
void CharFifo_Insert(TCharFifo *Fifo, u8 Value);

// Insert a Buffer of Len elements in the Fifo
void CharFifo_MultipleInsert(TCharFifo *Fifo, u8 *Buffer, u32 Len);

// Extract an element from the Fifo and return a pointer to it
u8 CharFifo_Extract(TCharFifo *Fifo);

// return the number of full elements in the fifo
u32 CharFifo_GetFull(TCharFifo *Fifo);

// return the number of free elements in the fifo
u32 CharFifo_GetFree(TCharFifo *Fifo);

#else

// Initialize a Fifo
inline void CharFifo_Init(TCharFifo *Fifo, u32 MaxElement, u8* DataBuffer)
{
  Fifo->Buffer = DataBuffer;
  Fifo->Max = MaxElement;
  Fifo->Full = 0;
  Fifo->First = 0;
  Fifo->Last = 0;
}

// Clear the content of a Fifo
inline void CharFifo_Clear(TCharFifo *Fifo)
{
  Fifo->Full = 0;
  Fifo->First = 0;
  Fifo->Last = 0;
}

// Insert a new element in the Fifo
inline void CharFifo_Insert(TCharFifo *Fifo, u8 Value)
{
  
  Fifo->Buffer[Fifo->Last]=Value;
  if(Fifo->Last >= Fifo->Max-1) {
    Fifo->Last = 0;
  } else {
    Fifo->Last++;
  }
  Fifo->Full++;
}

// Insert a Buffer of Len elements in the Fifo
inline void CharFifo_MultipleInsert(TCharFifo *Fifo, u8 *Buffer, u32 Len)
{
  u32 App, i;
  u8 *p;
  
  App=Len;
  if ((Fifo->Last+Len) >= Fifo->Max) {
    App=Fifo->Max-Fifo->Last;
  }
  p=&Fifo->Buffer[Fifo->Last];
  for (i=0; i<App; i++) {
    *p=*Buffer;
    p++;
    Buffer++;
  }
  App=Len-App;
  if (App==0) {
    Fifo->Last+=Len;
  } else {
    p=Fifo->Buffer;
    for (i=0; i<App; i++) {
      *p=*Buffer;
      p++;
      Buffer++;
    }
    Fifo->Last=App;
  }
  Fifo->Full+=Len;
}

// Extract an element from the Fifo and return a pointer to it
// if Validate==True then remove the element from the Fifo
inline u8 CharFifo_Extract(TCharFifo *Fifo)
{
  u8 App;
  
  App=Fifo->Buffer[Fifo->First];
  if (Fifo->First >= Fifo->Max-1) {
    Fifo->First = 0;
  } else {
    Fifo->First++;
  }
  Fifo->Full--;
  return App;
}

// return the number of full elements in the fifo
inline u32 CharFifo_GetFull(TCharFifo *Fifo)
{
  return Fifo->Full;
}

// return the number of free elements in the fifo
inline u32 CharFifo_GetFree(TCharFifo *Fifo)
{
  return Fifo->Max-Fifo->Full;
}

#endif

#endif  /* _CHAR_FIFO_H_ */

/******************** (C) COPYRIGHT 2008 STMicroelectronics *******************/

