/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* Author             : AST Robotics group
* Description        : header file
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __SERIAL_PROTOCOL__
#define __SERIAL_PROTOCOL__

/* Includes ------------------------------------------------------------------*/
#include "91x_lib.h"
#include "definitions.h"

/* Exported constants --------------------------------------------------------*/

#define CMD_Reply_Add               0x80 // Add this value to replys

#define CMD_PING                    0x01 // no param
#define CMD_Read_PresString         0x02 // no param
#define CMD_SelectDemo             	0x0A // u8 Demo
#define CMD_Reset	             	0x0F // u8 Demo

#define CMD_CameraGetState          0x10 // 0:Uniitialized, 1:Running, 2:Paused
#define CMD_CameraStart             0x11 // ImgFormat_U8, Width_U16, Height_U16, Framerate_U8, SensorMode_U8, HalfSysClock_U8, JpegCompr_U8, JpegDerat_U8
#define CMD_CameraStop              0x12 // no param
#define CMD_CameraGetImage          0x13 // FrameNumber_U16
#define CMD_CameraContinue          0x14 // no param

#define CMD_RGBCubeClear            0x21 // no param
#define CMD_RGBCubeSetClassR        0x22 // RG_U8, RB_U8, ClassID_U8
#define CMD_RGBCubeSetClassG        0x23 // GR_U8, GB_U8, ClassID_U8
#define CMD_RGBCubeSetClassB        0x24 // BR_U8, BG_U8, ClassID_U8
#define CMD_RGBCubeSetClassCube     0x25 // Rmin_U8, Rmax_U8, Gmin_U8, Gmax_U8, Bmin_U8, Bmax_U8, ClassID_U8

#define CMD_BlobSearchStart         0x31 // no param
#define CMD_BlobSearchStop          0x32 // no param
#define CMD_BlobSearchSetOptions    0x33 // GridSize_U8, SerchRect(Top_U8, Bottom_U8, Left_U8, Right_U8)
#define CMD_BlobSearchSetClassParam 0x34 // MinPer_U16,  MaxPer_U16, MinArea_U16, MaxArea_U16, MinCirc_U8, MaxCirc_U8, ClassID_U8
#define CMD_BlobSearchGetBlobs      0x35 // no param
#define CMD_BlobSearchFmiSaveBS     0x36 // no param
#define CMD_BlobSearchFmiLoadBS     0x37 // no param

#define CMD_HarrisGetPoints		    0x51 // no param
#define CMD_HarrisSettings		    0x52 // MaxHPt_U16, HarrisTh_U16, kh1_inv_U16

#define CMD_FastGetPoints		    0x61 // no param
#define CMD_FastSettings		    0x62 // MaxHPt_U16, HarrisTh_U16, kh1_inv_U16

#define DEMO_Undefined	 			0
#define DEMO_BlobSearch 			1
#define DEMO_FaceDetection 			2
#define DEMO_Harris 				3
#define DEMO_Jpeg 					4
#define DEMO_Fast					5

/* Exported types ------------------------------------------------------------*/

typedef struct {
  u32 Len;
  u8 Data[TMsg_MaxLen];
} TMsg;

/* Module private variables --------------------------------------------------*/
/* Exported macro ------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

// Build a Number from an array (LSB first)
u32 Deserialize(u8 *Source, u32 Len);

// Build an array from the u32 (LSB first)
void Serialize(u8 *Dest, u32 Source, u32 Len);

int ByteStuffCopyByte(u8 *Dest, u8 Source);
int ByteStuffCopy(u8 *Dest, TMsg *Source);
int ReverseByteStuffCopyByte(u8 *Source, u8 *Dest);
int ReverseByteStuffCopy(TMsg *Dest, u8 *Source);
void CHK_ComputeAndAdd(TMsg *Msg);
int CHK_CheckAndRemove(TMsg *Msg);

#endif /* __SERIAL_PROTOCOL__ */

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
