/******************** (C) COPYRIGHT 2006 STMicroelectronics ********************
 * File Name          : 91x_it.c
 * Author             : MCD Application Team
 * Date First Issued  : 05/18/2006 : Version 1.0
 * Description        : Main Interrupt Service Routines.
 ********************************************************************************
 *    This file can be used to describe all the exceptions subroutines
 *    that may occur within user application.
 *    When an interrupt happens, the software will branch automatically
 *    to the corresponding routine according to the interrupt vector
 *    loaded in the PC register.
 *    The following routines are all empty, user can write code for
 *    exceptions handlers and peripherals IRQ interrupts.
 ********************************************************************************
 * History:
 * 05/24/2006 : Version 1.1
 * 05/18/2006 : Version 1.0
 ********************************************************************************
 * THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 * CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME. AS
 * A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 * OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 * OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 * CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
 *******************************************************************************/
#include "91x_it.h"
#include "definitions.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "utils.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Extern variables ----------------------------------------------------------*/
extern u8 motor1_speed;
extern u8 servo_flag;
extern u8 motor2_speed;
extern u8 motor3_speed;
extern u8 servo_position;
extern void USB_Istr(void);
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/*******************************************************************************
 * Function Name  : Undefined_Handler
 * Description    : This function Undefined instruction exception.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void Undefined_Handler(void)
{
}
/*******************************************************************************
 * Function Name  : SWI_Handler
 * Description    : This function handles SW exception.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SWI_Handler(void)
{
}
/*******************************************************************************
 * Function Name  : Prefetch_Handler
 * Description    : This function handles preftetch abort exception.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void Prefetch_Handler(void)
{
}
/*******************************************************************************
 * Function Name  : Abort_Handler
 * Description    : This function handles data abort exception.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void Abort_Handler(void)
{
}
/*******************************************************************************
 * Function Name  : FIQ_Handler
 * Description    : This function handles FIQ exception.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void FIQ_Handler(void)
{
}
/*******************************************************************************
 * Function Name  : WDG_IRQHandler
 * Description    : This function handles the WDG interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void WDG_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : SW_IRQHandler
 * Description    : This function handles the SW interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SW_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : ARMRX_IRQHandler
 * Description    : This function handles the ARMRX interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void ARMRX_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : ARMTX_IRQHandler
 * Description    : This function handles the ARMTX interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void ARMTX_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : TIM0_IRQHandler
 * Description    : This function handles the TIM0 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void TIM0_IRQHandler(void)
{

	static u8 main_servo = 0;
	static u8 pulse_servo = 0;

	TIM_TypeDef *TIMx;

	TIMx = TIM0;

	if(TIM_GetFlagStatus(TIM0,TIM_FLAG_TO) == SET){
		TIM_ClearFlag(TIM0,TIM_FLAG_TO);

	}

	if(TIM_GetFlagStatus(TIM0,TIM_FLAG_OC1) == SET){
		TIM_ClearFlag(TIM0,TIM_FLAG_OC1);
		if(motor1_speed<135)
			uDelay(10);
		GPIO_WriteBit(MOTOR_GPIO,MOTOR1_Pin,Bit_RESET);
		//
		if(servo_flag !=STOP_SERVO){
			// the position should be changed or re-calibrated.
			pulse_servo++;
			switch (servo_position) {
			case 1:
				if(main_servo == 5)  //10 old_good //other 4
					GPIO_WriteBit(SERVO_GPIO,SERVO_Pin,Bit_RESET);
				break;
			case 2:
				if(main_servo == 3) //6 //other 2
					GPIO_WriteBit(SERVO_GPIO,SERVO_Pin,Bit_RESET);
				break;
			case 0:
				if(main_servo == 6) // 12 //other 4
					GPIO_WriteBit(SERVO_GPIO,SERVO_Pin,Bit_RESET);
				break;
			}
		}

	}

	if(TIM_GetFlagStatus(TIM0,TIM_FLAG_OC2) == SET){
		TIM_ClearFlag(TIM0,TIM_FLAG_OC2);

		GPIO_WriteBit(MOTOR_GPIO,MOTOR1_Pin,Bit_SET);

		if(servo_flag != STOP_SERVO){
			main_servo++;
			if(main_servo>=85){ // old value 170 //for other 67
				GPIO_WriteBit(SERVO_GPIO,SERVO_Pin,Bit_SET);
				main_servo=0;
			}
		}


		TIM_SetPulse(TIM0, TIM_OC2_Channel, 200);
		TIM_SetPulse(TIM0, TIM_PWM_OC1_Channel,motor1_speed);


		TIM_CounterCmd(TIM0,TIM_CLEAR);
	}

}
/*******************************************************************************
 * Function Name  : TIM1_IRQHandler
 * Description    : This function handles the TIM1 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void TIM1_IRQHandler(void)
{

	TIM_TypeDef *TIMx;

	TIMx = TIM1;

	if(TIM_GetFlagStatus(TIM1,TIM_FLAG_TO) == SET){
		TIM_ClearFlag(TIM1,TIM_FLAG_TO);

	}

	if(TIM_GetFlagStatus(TIM1,TIM_FLAG_OC1) == SET){
		TIM_ClearFlag(TIM1,TIM_FLAG_OC1);
		if(motor2_speed<135)
			uDelay(10);

		GPIO_WriteBit(MOTOR_GPIO,MOTOR2_Pin,Bit_RESET);

	}

	if(TIM_GetFlagStatus(TIM1,TIM_FLAG_OC2) == SET){
		TIM_ClearFlag(TIM1,TIM_FLAG_OC2);

		GPIO_WriteBit(MOTOR_GPIO,MOTOR2_Pin,Bit_SET);

		TIM_SetPulse(TIM1, TIM_OC2_Channel, 200);
		TIM_SetPulse(TIM1, TIM_PWM_OC1_Channel,motor2_speed);


		TIM_CounterCmd(TIM1,TIM_CLEAR);
	}

}
/*******************************************************************************
 * Function Name  : TIM2_IRQHandler
 * Description    : This function handles the TIM2 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void TIM2_IRQHandler(void)
{
	//	x24_TIM_SYNC_Handler();
}
/*******************************************************************************
 * Function Name  : TIM3_IRQHandler
 * Description    : This function handles the TIM3 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void TIM3_IRQHandler(void)
{

	TIM_TypeDef *TIMx;

	TIMx = TIM3;

	if(TIM_GetFlagStatus(TIM3,TIM_FLAG_TO) == SET){
		TIM_ClearFlag(TIM3,TIM_FLAG_TO);

	}

	if(TIM_GetFlagStatus(TIM3,TIM_FLAG_OC1) == SET){
		TIM_ClearFlag(TIM3,TIM_FLAG_OC1);
		if(motor3_speed< 135)
			uDelay(10);
		GPIO_WriteBit(MOTOR_GPIO,MOTOR3_Pin,Bit_RESET);

	}

	if(TIM_GetFlagStatus(TIM3,TIM_FLAG_OC2) == SET){
		TIM_ClearFlag(TIM3,TIM_FLAG_OC2);

		GPIO_WriteBit(MOTOR_GPIO,MOTOR3_Pin,Bit_SET);

		TIM_SetPulse(TIM3, TIM_OC2_Channel, 200);
		TIM_SetPulse(TIM3, TIM_PWM_OC1_Channel,motor3_speed);


		TIM_CounterCmd(TIM3,TIM_CLEAR);
	}

}
/*******************************************************************************
 * Function Name  : USBHP_IRQHandler
 * Description    : This function handles the USBHP interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USBHP_IRQHandler(void)
{
	CTR_HP();
}
/*******************************************************************************
 * Function Name  : USBLP_IRQHandler
 * Description    : This function handles the USBLP interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USBLP_IRQHandler(void)
{
	USB_Istr();
}
/*******************************************************************************
 * Function Name  : SCU_IRQHandler
 * Description    : This function handles the SCU interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SCU_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : ENET_IRQHandler
 * Description    : This function handles the ENET interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void ENET_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : DMA_IRQHandler
 * Description    : This function handles the DMA interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void DMA_IRQHandler(void)
{
	//  MY_ON();
	x24_DMA_IRQHandler();
	//  MY_OFF();
}
/*******************************************************************************
 * Function Name  : CAN_IRQHandler
 * Description    : This function handles the CAN interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void CAN_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : MC_IRQHandler
 * Description    : This function handles the MC interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void MC_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : ADC_IRQHandler
 * Description    : This function handles the ADC interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void ADC_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : UART0_IRQHandler
 * Description    : This function handles the UART0 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void UART0_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : UART1_IRQHandler
 * Description    : This function handles the UART1 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void UART1_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : UART2_IRQHandler
 * Description    : This function handles the UART2 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void UART2_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : I2C0_IRQHandler
 * Description    : This function handles the I2C0 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void I2C0_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : I2C1_IRQHandler
 * Description    : This function handles the I2C1 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void I2C1_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : SSP0_IRQHandler
 * Description    : This function handles the SSP0 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SSP0_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : SSP1_IRQHandler
 * Description    : This function handles the SSP1 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SSP1_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : LVD_IRQHandler
 * Description    : This function handles the LVD interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void LVD_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : RTC_IRQHandler
 * Description    : This function handles the RTC interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void RTC_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : WIU_IRQHandler
 * Description    : This function handles the WIU interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void WIU_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : EXTIT0_IRQHandler
 * Description    : This function handles the EXTIT0 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void EXTIT0_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : EXTIT1_IRQHandler
 * Description    : This function handles the EXTIT1 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void EXTIT1_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : EXTIT2_IRQHandler
 * Description    : This function handles the EXTIT2 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void EXTIT2_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : EXTIT3_IRQHandler
 * Description    : This function handles the EXTIT3 interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void EXTIT3_IRQHandler(void)
{
	x24_VSync_IRQ_Handler();
}
/*******************************************************************************
 * Function Name  : USBWU_IRQHandler
 * Description    : This function handles the USBWU interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USBWU_IRQHandler(void)
{
}
/*******************************************************************************
 * Function Name  : PFQBC_IRQHandler
 * Description    : This function handles the PFQBC interrupt request
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void PFQBC_IRQHandler(void)
{
}

/******************* (C) COPYRIGHT 2006 STMicroelectronics *****END OF FILE****/
