/* Standard include ----------------------------------------------------------*/
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "definitionsLib.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "RGBCube.h"
#include "blobsearch.h"
#include "math.h"
#include "Camera_Init.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
#define BUFFERSIZE_DEMO_BS	160*120*2
/* Private define ------------------------------------------------------------*/
#define TimeDiv 250000000
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
const u16 SyncOutPeriods[150] = { 9999, 8332, 7142, 6249,
		5555, 4999, 4545, 4166, 3846, 3571, 3333, 3125,
		2941, 2777, 2631, 2500, 2381, 2272, 2174, 2083,
		2000, 1923, 1852, 1786, 1724, 1666, 1613, 1562,
		1515, 1470, 1428, 1389, 1351, 1316, 1282, 1250,
		1219, 1190, 1163, 1136, 1111, 1087, 1064, 1042,
		1020, 1000, 980, 961, 943, 926, 909, 893, 877, 862,
		847, 833, 820, 806, 794, 781, 769, 757, 746, 735,
		725, 714, 704, 694, 685, 676, 667, 658, 649, 641,
		633, 625, 617, 610, 602, 595, 588, 581, 575, 568,
		562, 555, 549, 543, 538, 532, 526, 521, 515, 510,
		505, 500, 495, 490, 485, 481, 476, 472, 467, 463,
		459, 454, 450, 446, 442, 439, 435, 431, 427, 424,
		420, 417, 413, 410, 406, 403, 400, 397, 394, 391,
		388, 385, 382, 379, 376, 373, 370, 368, 365, 362,
		360, 357, 355, 352, 350, 347, 345, 342, 340, 338,
		336, 333 };
u8 *pImgBufferData;
u32 BufferSize;
Tx24_ImgJob Job[2], *CompletedJob = 0;
u8 CurrentJob;
u8 CamState = 0; // 0:Uniitialized, 1:Running, 2:Paused
TTimeStamp Tcam1, Tcam2;
vs32 OldErr = 0;
vs32 Ierr = 0, Nerr = 0;

/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/

void Init_BlobParameters(void) {
	// Define blobsearch constraints
	BlobSearchSetOption(5, 0, 120, 0, 160);
	// Define blobsearch geometry options
	//correct tihs
	BlobSearchSetColParam(1, 30, 2000, 50, 13000, 26, 153);
	BlobSearchSetColParam(2, 30, 2000, 50, 30000, 26, 153);

	//my green
	//day
	//	RGBCube_AddCube(0,6,4,8,0,1,1);

	//cloudly
	//day
	//	RGBCube_AddCube(0,6,4,8,1,3,1);
	//	RGBCube_SubCube(3,5,3,5,1,3,1);
	//end day
	RGBCube_AddCube(0, 6, 4, 6, 2, 3, 1);
	RGBCube_SubCube(3, 5, 2, 4, 1, 3, 1);
	RGBCube_SubCube(5, 7, 4, 6, 2, 4, 1);
	//	RGBCube_SubCube(3,5,4,6,2,4,1);
	//my orange
	RGBCube_AddCube(6, 8, 3, 5, 1, 3, 2);
	RGBCube_AddCube(5, 7, 1, 3, 0, 1, 2);

	/*night
	 RGBCube_AddCube(0,6,4,8,0,2,1);
	 RGBCube_SubCube(3,5,3,5,1,3,1);
	 night*/
}
void Init_Cam(void) {

	u8 ImgBufferData[2 * BUFFERSIZE_DEMO_BS];
	u8 RGBCubeData[4096];

	BufferSize = BUFFERSIZE_DEMO_BS;

	pImgBufferData = ImgBufferData;
	pRGBCubeData = RGBCubeData;

	RGBCube_Clear();
	BlobSearchInit();
	/*
	 * white balance & exposure compensation settings
	 */
	x24_WriteReg8(0x1380, 0); //wb
	x24_WriteReg8(0x1080, 2); //ae
	x24_WriteReg16(0x1095, 1000);
	x24_WriteReg16(0x109d, 240);
	x24_WriteRegF900(0x10a1, 1.0);

	u8 ImgFormat = x24_ImageFormat_RGB_565;
	u16 Width = 160;
	u16 Height = 120;
	u8 Framerate = 25;
	u8 SensorMode = x24_SensorMode_SVGA;
	u8 HalfSysClock = 80;
	u8 JpegCompr = 150;
	u8 JpegDerat = 10;
	CameraSetParam(ImgFormat, Width, Height, Framerate,
		SensorMode, HalfSysClock, JpegCompr, JpegDerat);
}

void GetCompletedJob(void) {
	if (Job[CurrentJob].State == x24_StateAcquisitionEnd) {
		CompletedJob = &Job[CurrentJob];
		CurrentJob ^= 1;
	}
}

void CameraSetParam(u8 ImgFormat, u16 Width, u16 Height,
		u8 Framerate, u8 SensorMode, u8 HalfSysClock,
		u8 JpegCompr, u8 JpegDerat) {
	Tx24_InitTypeDef x24_InitStruct;

	/* Initialize the x24 module */
	x24_StructInit(&x24_InitStruct);
	x24_InitStruct.SensorMode = SensorMode;
	x24_InitStruct.DesiredFrameRate_Num = Framerate * 250;
	x24_InitStruct.DesiredFrameRate_Den = 250;
	x24_InitStruct.ExtClockFreqMhz_Num = 8;
	x24_InitStruct.ExtClockFreqMhz_Den = 1;
	x24_InitStruct.JPEGClockDerate = JpegDerat;
	x24_InitStruct.SyncEnabled = 0;

	x24_InitStruct.ImageWidth = Width;
	x24_InitStruct.ImageHeight = Height;
	x24_InitStruct.ImageFormat = ImgFormat; // x24_ImageFormat_RGB_565 x24_ImageFormat_JPG
	x24_InitStruct.JPEGSqueezeValue = JpegCompr; // use 100 [800x600], 150 [1024x768], 220 [1600x1200]
	x24_InitStruct.SysClock = HalfSysClock * 2.0; // 35MHz; Range: [35:270] MHz  320*240@10Hz RGB_565

	x24_InitStruct.MemBlocksNum = 2;
	x24_InitStruct.ImgBlocksNum = 1;
	x24_InitStruct.ImgBlockSize = BufferSize;
	x24_InitStruct.pDataMemory = pImgBufferData;

	x24_InitStruct.ExposureCompensation
			= x24_biExposureCompensation;

	x24_Init(&x24_InitStruct);

	Job[0].pNext = &Job[1];
	Job[1].pNext = &Job[0];
	CurrentJob = 0;
	x24_GrabFrameStart(&Job[0]);

}
