/* Standard include ----------------------------------------------------------*/
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "definitionsLib.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "RGBCube.h"
#include "blobsearch.h"
#include "math.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Led_SM(void);
void Print_Int(int a, u8 * str);
void vectorByMatrix3x1(double A[][3], double x[3],
		double b[3]);
void vectorByMatrix4x1(double A[][4], double x[4],
		double b[4]);
void Matrix_Mult(double a1[][4], double a2[][4],
		double a3[][4]);
void Matrix_Mult3334(double a1[][3], double a2[][4],
		double a3[][4]);
void Matrix_Mult3443(double a1[][4], double a2[][3],
		double a3[][3]);
void Inverse(double A[][3], double X[][3]);
void vectorByMatrix3x1_s16(s16 A[][3], s16 x[3], s16 b[3]);
/* Private functions ---------------------------------------------------------*/

void Led_SM(void) {
	static int Count = 0;

	Count++;
	if (Count < 50000) {
		LED_ON(LEDR);
	} else {
		LED_OFF(LEDR);
		if (Count > 60000) {
			Count = 0;
		}
	}
}

void Print_Int(int a, u8 * str) {
	u8 myStr[10];
	USB_WriteString(str);
	USB_WriteString(":");
	Num2String(a, myStr);
	USB_WriteString(myStr);
	USB_WriteString("\r\n");
}
void vectorByMatrix3x1(double A[][3], double x[3],
		double b[3]) {

	b[0] = A[0][0] * x[0] + A[0][1] * x[1] + A[0][2] * x[2];
	b[1] = A[1][0] * x[0] + A[1][1] * x[1] + A[1][2] * x[2];
	b[2] = A[2][0] * x[0] + A[2][1] * x[1] + A[2][2] * x[2];

}
void vectorByMatrix3x1_s16(s16 A[][3], s16 x[3], s16 b[3]) {
	b[0] = A[0][0] * x[0] / 100 + A[0][1] * x[1] / 100
			+ A[0][2] * x[2] / 100;
	b[1] = A[1][0] * x[0] / 100 + A[1][1] * x[1] / 100
			+ A[1][2] * x[2] / 100;
	b[2] = A[2][0] * x[0] / 100 + A[2][1] * x[1] / 100
			+ A[2][2] * x[2] / 100;

}
void vectorByMatrix4x1(double A[][4], double x[4],
		double b[4]) {

	b[0] = A[0][0] * x[0] + A[0][1] * x[1] + A[0][2] * x[2]
			+ A[0][3] * x[3];
	b[1] = A[1][0] * x[0] + A[1][1] * x[1] + A[1][2] * x[2]
			+ A[1][3] * x[3];
	b[2] = A[2][0] * x[0] + A[2][1] * x[1] + A[2][2] * x[2]
			+ A[2][3] * x[3];
	b[3] = A[3][0] * x[0] + A[3][1] * x[1] + A[3][2] * x[2]
			+ A[3][3] * x[3];

}
void Matrix_Mult(double a1[][4], double a2[][4],
		double a3[][4]) {
	int i = 0;
	int j = 0;
	int k = 0;
	int a = 4;
	int b = 4;
	int c = 4;

	for (i = 0; i < a; i++)
		for (j = 0; j < b; j++)
			for (k = 0; k < c; k++)
				a3[i][j] += a1[i][k] * a2[k][j];
}
void Matrix_Mult3334(double a1[][3], double a2[][4],
		double a3[][4]) {
	int i = 0;
	int j = 0;
	int k = 0;
	for (i = 0; i < 3; i++)
		for (j = 0; j < 4; j++)
			for (k = 0; k < 3; k++)
				a3[i][j] += a1[i][k] * a2[k][j];
}

void Matrix_Mult3443(double a1[][4], double a2[][3],
		double a3[][3]) {
	int i = 0;
	int j = 0;
	int k = 0;
	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			for (k = 0; k < 4; k++)
				a3[i][j] += a1[i][k] * a2[k][j];
}
void Inverse(double A[][3], double X[][3]) {
	float B[3][3];//the transpose of a matrix A
	float C[3][3];//the adjunct matrix of transpose of a matrix A not adjunct of A
	int i, j;
	float x, n = 0;//n is the determinant of A


	for (i = 0, j = 0; j < 3; j++) {
		if (j == 2)
			n += A[i][j] * A[i + 1][0] * A[i + 2][1];
		else if (j == 1)
			n += A[i][j] * A[i + 1][j + 1] * A[i + 2][0];
		else
			n += A[i][j] * A[i + 1][j + 1]
					* A[i + 2][j + 2];
	}
	for (i = 2, j = 0; j < 3; j++) {
		if (j == 2)
			n -= A[i][j] * A[i - 1][0] * A[i - 2][1];
		else if (j == 1)
			n -= A[i][j] * A[i - 1][j + 1] * A[i - 2][0];
		else
			n -= A[i][j] * A[i - 1][j + 1]
					* A[i - 2][j + 2];
	}

	if (n != 0)
		x = 1.0 / n;
	else {
	}
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {

			B[i][j] = A[j][i];

		}
	}

	C[0][0] = B[1][1] * B[2][2] - (B[2][1] * B[1][2]);
	C[0][1] = (-1) * (B[1][0] * B[2][2] - (B[2][0]
			* B[1][2]));
	C[0][2] = B[1][0] * B[2][1] - (B[2][0] * B[1][1]);

	C[1][0] = (-1)
			* (B[0][1] * B[2][2] - B[2][1] * B[0][2]);
	C[1][1] = B[0][0] * B[2][2] - B[2][0] * B[0][2];
	C[1][2] = (-1)
			* (B[0][0] * B[2][1] - B[2][0] * B[0][1]);

	C[2][0] = B[0][1] * B[1][2] - B[1][1] * B[0][2];
	C[2][1] = (-1)
			* (B[0][0] * B[1][2] - B[1][0] * B[0][2]);
	C[2][2] = B[0][0] * B[1][1] - B[1][0] * B[0][1];

	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			X[i][j] = C[i][j] * x;

		}
	}

}

