/* Standard include ----------------------------------------------------------*/
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "definitionsLib.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "math.h"
#include "Motor_Init.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
u8 control_hit;
u8 servo_position = 0;
u8 servo_flag = 0;

u8 motor1_speed;
u8 motor2_speed;
u8 motor3_speed;

/* Private function prototypes -----------------------------------------------*/
void Init_Motors(void);
void Calculate_Order(s16* speed, u8 size);
void Position_Camera(u8 position);
void Set_Directions(u8 Motor1, u8 Motor2, u8 Motor3);
void Stop_Motor(u8 Motor);
void Bumpers(u8 bumper[6]);
void Set_Speed(s16* speed, u8 mod);
void Stop_Servo(void);
void Turn(u8 direction);
/* Private functions ---------------------------------------------------------*/

void Set_Speed(s16* speed, u8 mod) {

	u8 direction[NUM_MOTORS];
	u8 k;

	for (k = 0; k < NUM_MOTORS; k++) {
		direction[k] = (speed[k] > 0) ? 1 : 0, speed[k]
				=Abs(speed[k]);

	}

	for (k = 0; k < NUM_MOTORS; k++) {

		if (speed[k] == 0) {
			Stop_Motor(k);
		}
		if (speed[k] > 2500)
			speed[k] = 2500;
	}

	for (k = 0; k < NUM_MOTORS; k++) {
		if (speed[k] != 0)
			speed[k] = (speed[k] / 75 * mod) + 90;
	}

	Set_Directions(direction[0], direction[1], direction[2]);

	motor1_speed = (u8) speed[0];
	motor2_speed = (u8) speed[1];
	motor3_speed = (u8) speed[2];
#ifdef NO_DEBUG
	Print_Int(motor1_speed,"m1 applied");
	Print_Int(motor2_speed,"m2 applied");
	Print_Int(motor3_speed,"m3 applied");
#endif
}

void Stop_Motor(u8 Motor) {
	switch (Motor) {
	case 0: //stop 1
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_B, Bit_RESET);
		break;
	case 1: //stop 2
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_B, Bit_RESET);
		break;
	case 2: //stop 3
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_B, Bit_RESET);
		break;
	case 3: //stop all
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_B, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_B, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_B, Bit_RESET);
		break;
	}
}

void Set_Directions(u8 Motor1, u8 Motor2, u8 Motor3) {
	if (Motor2 == FORWARD) {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_A, Bit_SET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_B, Bit_RESET);
	} else {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR2_DIRECTION_B, Bit_SET);
	}
	if (Motor1 == FORWARD) {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_A, Bit_SET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_B, Bit_RESET);
	} else {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR1_DIRECTION_B, Bit_SET);
	}
	if (Motor3 == FORWARD) {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_A, Bit_SET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_B, Bit_RESET);
	} else {
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_A, Bit_RESET);
		GPIO_WriteBit(MOTOR_DIRECTION_GPIO,
			MOTOR3_DIRECTION_B, Bit_SET);
	}

}

void Calculate_Order(s16* speed, u8 size) {

	u8 pass = 1;
	u8 sorted = 0;
	u8 i;
	while ((!sorted) && (pass < size)) {
		sorted = 1;
		for (i = 0; i < size - pass; i++) {
			if (speed[i] > speed[i + 1]) {
				int temp = speed[i];
				speed[i] = speed[i + 1];
				speed[i + 1] = temp;
				sorted = 0;

			}
		}
		pass++;
	}
}
void Stop_Servo() {
	servo_flag = STOP_SERVO;
}

void Init_Motors() {
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = MOTOR1_Pin | MOTOR2_Pin
			| MOTOR3_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(MOTOR_GPIO, &GPIO_InitStructure);

	/* Config Servo PIN */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = SERVO_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(SERVO_GPIO, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = MOTOR1_DIRECTION_A
			| MOTOR1_DIRECTION_B | MOTOR2_DIRECTION_A
			| MOTOR2_DIRECTION_B | MOTOR3_DIRECTION_A
			| MOTOR3_DIRECTION_B;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(MOTOR_DIRECTION_GPIO, &GPIO_InitStructure);

	TIM_InitTypeDef TIM_InitStructure;

	TIM_StructInit(&TIM_InitStructure);
	TIM_InitStructure.TIM_Mode = TIM_OCM_CHANNEL_12; //for both use 12
	TIM_InitStructure.TIM_OC1_Modes = TIM_TIMING;
	TIM_InitStructure.TIM_OC2_Modes = TIM_TIMING;
	TIM_InitStructure.TIM_Clock_Source = TIM_CLK_APB;
	TIM_InitStructure.TIM_Prescaler = 47; //127
	TIM_InitStructure.TIM_Pulse_Length_1 = 50;
	TIM_InitStructure.TIM_Pulse_Length_2 = 200;
	TIM_InitStructure.TIM_Pulse_Level_1 = TIM_HIGH;
	TIM_InitStructure.TIM_Pulse_Level_2 = TIM_HIGH;
	TIM_InitStructure.TIM_Period_Level = TIM_LOW;

	TIM_Init(TIM0, &TIM_InitStructure);
	TIM_Init(TIM1, &TIM_InitStructure);
	TIM_Init(TIM3, &TIM_InitStructure);

	/*Enable TIM0 Output Compare1 interrupt*/
	TIM_ITConfig(TIM0, TIM_IT_TO | TIM_IT_OC1 | TIM_IT_OC2,
		ENABLE);
	/*Enable TIM1 Output Compare1 interrupt*/
	TIM_ITConfig(TIM1, TIM_IT_TO | TIM_IT_OC1 | TIM_IT_OC2,
		ENABLE);
	/*Enable TIM3 Output Compare1 interrupt*/
	TIM_ITConfig(TIM3, TIM_IT_TO | TIM_IT_OC1 | TIM_IT_OC2,
		ENABLE);

	VIC_Config(TIM0_ITLine, VIC_IRQ, 9);
	VIC_ITCmd(TIM0_ITLine, ENABLE);

	VIC_Config(TIM1_ITLine, VIC_IRQ, 10);
	VIC_ITCmd(TIM1_ITLine, ENABLE);

	VIC_Config(TIM3_ITLine, VIC_IRQ, 11);
	VIC_ITCmd(TIM3_ITLine, ENABLE);

	/*Start*/
	TIM_CounterCmd(TIM0, TIM_START);
	TIM_CounterCmd(TIM1, TIM_START);
	TIM_CounterCmd(TIM3, TIM_START);

	u8 i;
	s16 speed[3];
	for (i = 0; i < NUM_MOTORS; i++) {
		speed[i] = 0;
	}

	Set_Speed(speed, CLOSE);
}

void Position_Camera(u8 position) {

	servo_flag = 0;
	servo_position = position;

}
void Init_Bumpers() {
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_Init(GPIO5, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_Init(GPIO1, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_Init(GPIO4, &GPIO_InitStructure);

}
void Bumpers(u8 bumper[6]) {
	control_hit = 0;

	if (GPIO_ReadBit(GPIO5, BUMPER_FRONT_LEFT) == Bit_SET) {
		bumper[0] = 1;
	}
	if (GPIO_ReadBit(GPIO4, BUMPER_FRONT_RIGHT) == Bit_SET) {
		bumper[4] = 1;
	}
	if (GPIO_ReadBit(GPIO1, BUMPER_LEFT) == Bit_SET) {
		bumper[2] = 1;
	}
	if (GPIO_ReadBit(GPIO5, BUMPER_RIGHT) == Bit_SET) {
		bumper[1] = 1;
	}
	if (GPIO_ReadBit(GPIO1, BUMPER_BACK_LEFT) == Bit_SET) {
		bumper[3] = 1;
	}

	if (GPIO_ReadBit(GPIO4, BUMPER_BACK_RIGHT) == Bit_SET) {
		bumper[5] = 1;
	}
	u8 i;

	for (i = 0; i < 6; i++) {
		if (bumper[i] == 1) {
			control_hit = 1;
#ifdef NO_DEBUG
			Print_Int(i,"no");
			Print_Int(bumper[i],"Bumper");
#endif
		}
	}
}

void Turn(u8 direction) {

	s16 left_speed[3] = { 550, 550, 550 };
	s16 right_speed[3] = { -550, -550, -550 };

	if (direction == LEFT) {
		Set_Speed(left_speed, NORMAL);

	} else {
		Set_Speed(right_speed, NORMAL);
	}
	MSDelay(100);
	Stop_Motor(3);
	MSDelay(100);
}
