/* Standard include ----------------------------------------------------------*/
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "definitionsLib.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "RGBCube.h"
#include "blobsearch.h"
#include "math.h"
#include "Misc.h"
#include "Vision.h"
#include "Camera_Init.h"
#include "Motion.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//#define PRINT_ON

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
u8 *pImgBufferData;
u32 BufferSize;
double BlobSample[10][2];
u8 index = 0;
extern Tx24_ImgJob Job[2];
extern CurrentJob;
extern u8 control_hit;
/* Private function prototypes -----------------------------------------------*/
//see header
/* Private functions ---------------------------------------------------------*/
void Blob_Search(void) {
	if (Job[CurrentJob].State == x24_StateAcquisitionEnd) { // Image is acquired
		MY_ON();
		CompletedJob = &Job[CurrentJob];
		BlobSearch((u16*) CompletedJob->pFirstBlock->pData);
		MY_OFF();
		CurrentJob ^= 1;
	}
}

void CalculatePositionTop(TPoint point, s16 result[3]) {
	//Homography for servo_top
	double K[3][3] = { { 191.7146, 0, 80.3591 }, { 0,
			191.2730, 61.2765 }, { 0, 0, 1 } },
			T_cr[3][4] = { { -0.0755, -0.9948, 0.0683,
					-8.6194 }, { -0.1867, -0.0531, -0.9810,
					271.8015 }, { 0.9795, -0.0868, -0.1817,
					-72.3125 } }, C[3][4] = {
					{ 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0,
							0, 0 } },
			H[3][3] = { { 0, 0, 0 }, { 0, 0, 0 },
					{ 0, 0, 0 } },
			InvH[3][3] = { { 0, 0, 0 }, { 0, 0, 0 }, { 0,
					0, 0 } }, t_[4][3] = { { 1, 0, 0 }, {
					0, 1, 0 }, { 0, 0, 0 }, { 0, 0, 1 } };

	Matrix_Mult3334(K, T_cr, C);

	Matrix_Mult3443(C, t_, H);

	Inverse(H, InvH);

	double position[3] = { 0, 0, 0 };
	double image_point[3] = { point.X, point.Y, 1 };

	vectorByMatrix3x1(InvH, image_point, position);

	result[0] = (s16) (position[0] / position[2]);
	result[1] = (s16) (position[1] / position[2]);
	result[2] = (s16) (position[2] / position[2]);

}
void CalculatePositionGround(TPoint point, s16 result[3]) {
	//Homography for servo_top
	double K[3][3] = { { 191.7146, 0, 80.3591 }, { 0,
			191.2730, 61.2765 }, { 0, 0, 1 } },
			T_cr[3][4] = { { -0.0160, -0.9987, 0.0478,
					-0.6271 }, { -0.6634, -0.0252, -0.7478,
					245.6112 }, { 0.7481, -0.0437, -0.6622,
					44.2694 } }, C[3][4] = {
					{ 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0,
							0, 0 } },
			H[3][3] = { { 0, 0, 0 }, { 0, 0, 0 },
					{ 0, 0, 0 } },
			InvH[3][3] = { { 0, 0, 0 }, { 0, 0, 0 }, { 0,
					0, 0 } }, t_[4][3] = { { 1, 0, 0 }, {
					0, 1, 0 }, { 0, 0, 0 }, { 0, 0, 1 } };

	Matrix_Mult3334(K, T_cr, C);

	Matrix_Mult3443(C, t_, H);

	Inverse(H, InvH);

	double position[3] = { 0, 0, 0 };
	double image_point[3] = { point.X, point.Y, 1 };

	vectorByMatrix3x1(InvH, image_point, position);

	result[0] = (s16) (position[0] / position[2]);
	result[1] = (s16) (position[1] / position[2]);
	result[2] = (s16) (position[2] / position[2]);

}

void CalculateBallPosition(u8 X, u8 Y, u8 Dpx,
		double result[4]) {

	double image_point[3] = { X, Y, 1 };
	double inv_K[3][3] = { { 0.005216086284917, 0,
			-0.419160051539105 }, { 0, 0.005228129700905,
			-0.320361489617536 }, { 0, 0, 1 } };

	double position[3] = { 0, 0, 0 };

	vectorByMatrix3x1(inv_K, image_point, position);

	double norm;
	norm = sqrt((position[0] * position[0]) + (position[1]
			* position[1]) + (position[2] * position[2]));

	position[0] = position[0] / norm;
	position[1] = position[1] / norm;
	position[2] = position[2] / norm;

	int X_axis = 300;
	int Y_axis = -150;
	int Dreal = 62; //red ball w 6.2 cm diam

	double fstar = 160.9969;

	double Pc[4] = { (fstar * Dreal / Dpx) * position[0],
			(fstar * Dreal / Dpx) * position[1], (fstar
					* Dreal / Dpx) * position[2], 1 };

	double T_wr[4][4] = { { 1, 0, 0, X_axis }, { 0, 1, 0,
			Y_axis }, { 0, 0, 1, 0 }, { 0, 0, 0, 1 } };

	double T_wc[4][4] = { { 0.0012, -0.3587, 0.9334,
			-350.8669 }, { -1.0000, -0.0039, -0.0002,
			140.5637 }, { 0.0037, -0.9334, -0.3587,
			203.8752 }, { 0, 0, 0, 1.0000 } };

	double inv_T[4][4] = { 0, 0, 0, 0 };
	Matrix_Mult(T_wr, T_wc, inv_T);

	vectorByMatrix4x1(inv_T, Pc, result);
}

