/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
 * File Name          : main.c
 * Author             : AST Robotics group
 * Date First Issued  : 11 May 2007 : Version 1.0
 * Description        : Main program body
 ********************************************************************************
 * History:
 * 28 May 2007 : Version 1.2
 * 11 May 2007 : Version 1.0
 ********************************************************************************
 * THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
 * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 * AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
 * INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
 * CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
 * INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
 *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "91x_lib.h"
#include "definitions.h"
#include "CamInt_x24.h"
#include "utils.h"
#include "math.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "definitionsLib.h"
#include "Misc.h"
#include "Camera_Init.h"
#include "Motor_Init.h"
#include "blobsearch.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
extern u8 control_hit;
/* Private function prototypes -----------------------------------------------*/

void SCU_Configuration(void);
void USB_Configuration(void);
void GPIO_Configuration(void);

/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
 * Function Name  : main
 * Description    : Main program
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
int main(void) {

#ifdef DEBUG
	debug();
#endif

	/* Configure the system clocks */
	SCU_Configuration();

	/* Configure the GPIO ports */
	GPIO_Configuration();

	/* Configure x24 Camera */
	x24_HwConfig(8000000);
	x24_SendPatches();

	/* Configure the USB */
	USB_Configuration();
	USB_Init();

	LED_ON(LEDR);
	MSDelay(200);
	LED_OFF(LEDR);
	MSDelay(200);
	LED_ON(LEDR);
	MSDelay(200);
	LED_OFF(LEDR);
	MSDelay(200);
	LED_ON(LEDR);
	MSDelay(200);
	LED_OFF(LEDR);

	/* To initialize camera parameters*/
	Init_Cam();

	/* To initialize motors, configure pins and PWM values*/
	Init_Motors();
	/* To arrange camera head position
	 * available position SERVO_TOP,SERVO_MID,SERVO_BOTTOM
	 * */
	Position_Camera(SERVO_TOP);
	/* To initialize bumpers, and configure pins */
	Init_Bumpers();

	MSDelay(2000);
	/* To define blob color & geometry */
	Init_BlobParameters();

	/* the initial values */
	bool search_direction = TRUE;
	u8 game_end = 0;
	bool blob_found = FALSE;
	bool color_done = FALSE;
	u8 cam_position = SERVO_TOP;
	u8 counter = 0;
	u8 color = 1;
	s16 speed[3];
	u8 replay_counter = 0;
	while (1) {
		/*reading the bumpers values */
		u8 check_bumpers[6] = { 0 };
		Bumpers(check_bumpers);
		/*to blink led*/
		Led_SM();
		/* controlling whether the first target acquired */
		if (color_done == TRUE) {
			/* to check whether game_end satified */
			//			if(game_end == 0){//IF COLOR <3
			//			if(color <3){
			Stop_Motor(3);
			MSDelay(100);
			cam_position = SERVO_TOP;
			Position_Camera(SERVO_TOP);
			//				color=2;//
			color++;
			color_done = FALSE;
			blob_found = FALSE;
			//			}

			//			game_end= game_end + 1;//
			//			if(game_end == 2){//
			if (color > 2) {
				if (replay_counter < 5) {
					replay_counter++;
					color = 1;
				} else {
					Stop_Motor(3);
					while (1)
						;
				}

			}

		} else { //else of color_1_done
			if (control_hit == 1) { //checking collision
				//do something
				if (check_bumpers[0] == 1
						|| check_bumpers[4] == 1
						|| check_bumpers[3] == 1
						|| check_bumpers[5] == 1)//hit front
				{//go back
					speed[0] = 1000;
					speed[1] = -1000;
					speed[2] = 0;
					Set_Speed(speed, NORMAL);
					MSDelay(100);

					//				}else if(check_bumpers[3]==1 || check_bumpers[5]==1)//hit back
					//				{//go forward
					//					speed[0]=-1000;
					//					speed[1]=1000;
					//					speed[2]=0;
					//					Set_Speed(speed,NORMAL);
					//					MSDelay(100);
				} else if (check_bumpers[1] == 1)//hit right
				{//go left
					speed[0] = -750;
					speed[1] = -750;
					speed[2] = 1200;
					Set_Speed(speed, NORMAL);
					MSDelay(100);
				} else if (check_bumpers[2] == 1)//hit left
				{//go right
					speed[0] = 750;
					speed[1] = 750;
					speed[2] = -1200;
					Set_Speed(speed, NORMAL);
					MSDelay(100);
				}

#ifdef NO_DEBUG
				Print_Int(speed[0],"b1 motor send:");
				Print_Int(speed[1],"b2 motor send:");
				Print_Int(speed[2],"b3 motor send:");
#endif
			} else { //control hit else
#ifdef NO_DEBUG
				Print_Int(control_hit,"hit?");
#endif
				Blob_Search();
#ifdef NO_DEBUG
				Print_Int(BlobCount," blobs found...\n");
#endif

				TPoint n_point;
				u8 k;
				s8 myBlob;
				s16 maxArea;
				maxArea = 0;
				myBlob = -1;//arif =0
				/*
				 * among the available blob the blob with the
				 * biggest size is selected as the blob
				 */
				for (k = 0; k < BlobCount; k++) {
					if (Blobs[k].ColorSetID == color) {
						if (maxArea < Blobs[k].Area) {
							maxArea = Blobs[k].Area;
							myBlob = k;
						}
					}
				}

				//if(BlobCount == 0){
				if (myBlob < 0) {//arif
					//random search
#ifdef NO_DEBUG
					USB_WriteString("Searching for a blob...\n");
#endif
					/* in order to prevent continue turning to search for blob
					 * counter introduced.
					 */
					if (blob_found == TRUE) {
						counter++;
					} else if (counter > 3 || blob_found
							== FALSE) { //arif
						/*search_direction changes the turning side to left to
						 * right or vice versa
						 */
						if (search_direction == TRUE) {
							Turn(RIGHT);
						} else {
							Turn(LEFT);
						}
						blob_found = FALSE;
					}
				} else {

					//					u8 k;
					//					u8 myBlob;
					//					s16 maxArea;
					//					maxArea = Blobs[0].Area;
					//					myBlob = 0;
					//					/*
					//					 * among the available blob the blob with the
					//					 * biggest size is selected as the blob
					//					 */
					//					for(k=0;k<BlobCount-1;k++){
					//						if(Blobs[k].ColorSetID == color) {
					//						if(Blobs[k].Area < Blobs[k+1].Area){
					//							maxArea = Blobs[k+1].Area;
					//							myBlob = k+1;
					//						}
					//						}
					//					}
					/*
					 * search direction is reversed
					 */
					search_direction = !search_direction;

					/*
					 * controlling if the blob is with the correct color
					 */
					//					if(Blobs[myBlob].ColorSetID == color){
					blob_found = TRUE;
					counter = 0;
					n_point.X = Blobs[myBlob].Centroid.X;
					n_point.Y
							= Blobs[myBlob].OuterBox.Bottom;
					s16 position[3];

					/*
					 * for different camera positions different
					 * calibrations and motion mechanism is used
					 */

					if (cam_position == SERVO_TOP) {
						CalculatePositionTop(n_point,
							position);
#ifdef NO_DEBUG
						Print_Int(n_point.X,"X in cam coordinates");
						Print_Int(n_point.Y,"Y in cam coordinates");
						Print_Int(position[0],"X in real coordinates");
						Print_Int(position[1],"Y in real coordinates");
#endif
						double alpha = 0;
						alpha = atan2(position[1],
							position[0]);
						s8 omega_modifid = alpha * 100;
						Triskar(position, omega_modifid,
							speed);

						if (Abs(position[0]) > 1200) {
							speed[2] = speed[2] / 2;
							Set_Speed(speed, FAR);
						} else if (Abs(position[0]) > 700) {
							Set_Speed(speed, NORMAL);
						} else {
							cam_position = SERVO_MID;
							Position_Camera(SERVO_MID);
							//								speed[2]= speed[2] / 2;
							//								Set_Speed(speed,CLOSE);
						}
#ifdef NO_DEBUG
						Print_Int(speed[0],"1 motor send:");
						Print_Int(speed[1],"2 motor send:");
						Print_Int(speed[2],"3 motor send:");
#endif

					}//servo_top
					else {//servo_mid
						CalculatePositionGround(n_point,
							position);
						double alpha = 0;
						alpha = atan2(position[1],
							position[0]);
						s8 omega_modifid = alpha * 100;
						Triskar(position, omega_modifid,
							speed);
						speed[2] = speed[2] / 2;
						Set_Speed(speed, CLOSE);
						if (n_point.Y > 115) {
							color_done = TRUE;
						}
					}
					//					}//color id
					//					else{
					//						blob_found = FALSE;
					//					}
				}//blob

			}//else of control hit
		}

	}
}

/*******************************************************************************
 * Function Name  : SCU_Configuration
 * Description    : Configures the system clocks.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void SCU_Configuration(void) {
	/* Initialize PLL */
	SCU_MCLKSourceConfig(SCU_MCLK_OSC);
	SCU_FMICLKDivisorConfig(SCU_FMICLK_Div1);
	FMI_Config(FMI_READ_WAIT_STATE_2,
		FMI_WRITE_WAIT_STATE_0, FMI_PWD_ENABLE,
		FMI_LVD_ENABLE, FMI_FREQ_HIGH);
	SCU_RCLKDivisorConfig(SCU_RCLK_Div1);
	SCU_HCLKDivisorConfig(SCU_HCLK_Div1);
	SCU_PCLKDivisorConfig(SCU_PCLK_Div2);
	SCU_BRCLKDivisorConfig(SCU_BRCLK_Div1);
	SCU_PLLFactorsConfig(192, 25, 2); /* PLL = 96 MHz */
	SCU_PLLCmd(ENABLE); /* PLL Enabled  */
	SCU_MCLKSourceConfig(SCU_MCLK_PLL); /* MCLK = PLL   */

	SCU_PFQBCCmd(ENABLE);

	/* Enable GPIO 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 Clocks */
	SCU_APBPeriphClockConfig(__GPIO0, ENABLE);
	GPIO_DeInit(GPIO0);
	SCU_APBPeriphClockConfig(__GPIO1, ENABLE);
	GPIO_DeInit(GPIO1);
	SCU_APBPeriphClockConfig(__GPIO2, ENABLE);
	GPIO_DeInit(GPIO2);
	SCU_APBPeriphClockConfig(__GPIO3, ENABLE);
	GPIO_DeInit(GPIO3);
	SCU_APBPeriphClockConfig(__GPIO4, ENABLE);
	GPIO_DeInit(GPIO4);
	SCU_APBPeriphClockConfig(__GPIO5, ENABLE);
	GPIO_DeInit(GPIO5);
	SCU_APBPeriphClockConfig(__GPIO6, ENABLE);
	GPIO_DeInit(GPIO6);
	SCU_APBPeriphClockConfig(__GPIO7, ENABLE);
	GPIO_DeInit(GPIO7);
	SCU_APBPeriphClockConfig(__GPIO8, ENABLE);
	GPIO_DeInit(GPIO8);
	SCU_APBPeriphClockConfig(__GPIO9, ENABLE);
	GPIO_DeInit(GPIO9);

	/* Enable VIC clock */
	SCU_AHBPeriphClockConfig(__VIC, ENABLE);
	VIC_DeInit();

	/* Enable WIU clock */
	SCU_APBPeriphClockConfig(__WIU, ENABLE);
	WIU_DeInit();

	/* Enable WIU clock */
	SCU_APBPeriphClockConfig(__I2C0, ENABLE);

	/* Enable DMA clock */
	SCU_AHBPeriphClockConfig(__DMA, ENABLE);
	DMA_DeInit();

	/* Enable TIM0123 clock */
	SCU_APBPeriphClockConfig(__TIM01, ENABLE);
	TIM_DeInit(TIM0);
	TIM_DeInit(TIM1);
	SCU_APBPeriphClockConfig(__TIM23, ENABLE);
	TIM_DeInit(TIM2);
	TIM_DeInit(TIM3);
	SCU_TIMPresConfig(SCU_TIM01, 4800); // ~10KHz
	SCU_TIMExtCLKCmd(SCU_TIM01, DISABLE); // Disable external pin
	SCU_TIMPresConfig(SCU_TIM23, 4800); // ~10KHz
	SCU_TIMExtCLKCmd(SCU_TIM23, DISABLE); // Disable external pin

	SCU_APBPeriphClockConfig(__I2C0, ENABLE);
	I2C_DeInit(I2C0);

	SCU_AHBPeriphClockConfig(__FMI, ENABLE);
	SCU_AHBPeriphReset(__FMI, DISABLE);
}

/*******************************************************************************
 * Function Name  : USB_Configuration
 * Description    : Configures the USB
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void USB_Configuration(void) {
	GPIO_InitTypeDef GPIO_InitStructure;

	/* USB clock = MCLK/2 = 48MHz */
	SCU_USBCLKConfig(SCU_USBCLK_MCLK2);
	/* Enable USB clock */
	SCU_AHBPeriphClockConfig(__USB, ENABLE);
	SCU_AHBPeriphReset(__USB, DISABLE);
	SCU_AHBPeriphClockConfig(__USB48M, ENABLE);

	/* Configure USB D+ PullUp pin */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = USB_Dp_PullUp_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_OpenCollector;//GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(USB_Dp_PullUp_GPIO, &GPIO_InitStructure);
	USB_Dp_PullUp_OFF();
	MSDelay(100);
	USB_Dp_PullUp_ON();

	USB_endPointConf();

	VIC_Config(USBLP_ITLine, VIC_IRQ, USB_Priority);
	VIC_ITCmd(USBLP_ITLine, ENABLE);
}

/*******************************************************************************
 * Function Name  : GPIO_Configuration
 * Description    : Configures the different GPIO ports.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void GPIO_Configuration(void) {
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure LEDR */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = LEDR_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(LEDR_GPIO, &GPIO_InitStructure);

	/* Configure MY_GPIO */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = MY_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	GPIO_Init(MY_GPIO, &GPIO_InitStructure);
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
