/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : serial_protocol.c
* Author             : AST Robotics group
* Date First Issued  : 25 Sept 2008: Version 1.0
* Description        : This file implements the usb-webcam demo
********************************************************************************
* History:
* 25 Sept 2008: Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "serial_protocol.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

u32 Deserialize(u8 *Source, u32 Len)
// Build a Number from an array (LSB first)
{
  u32 app;
  app=Source[--Len];
  while(Len>0) {
    app<<=8;
    app+=Source[--Len];
  }
  return app;
}

void Serialize(u8 *Dest, u32 Source, u32 Len)
// Build an array from the u32 (LSB first)
{
  int i;
  for (i=0; i<Len; i++) {
    Dest[i] = Source & 0xFF;
    Source>>=8;
  } 
}

int ByteStuffCopyByte(u8 *Dest, u8 Source)
{
  switch(Source) {
    case TMsg_EOF:
      Dest[0]=TMsg_BS;
      Dest[1]=TMsg_BS_EOF;
      return 2;
    case TMsg_BS:
      Dest[0]=TMsg_BS;
      Dest[1]=TMsg_BS;
      return 2;
    default:
      Dest[0]=Source;
      return 1;
  }
}

int ByteStuffCopy(u8 *Dest, TMsg *Source)
{
  int i, Count;
  
  Count=0;
  for (i=0; i<Source->Len; i++) {
    Count+=ByteStuffCopyByte(&Dest[Count], Source->Data[i]);
  }
  Dest[Count]=TMsg_EOF;
  Count++;
  return Count;
}

int ReverseByteStuffCopyByte(u8 *Source, u8 *Dest)
{
  if (Source[0] == TMsg_BS) {
    if (Source[1] == TMsg_BS) {
      *Dest=TMsg_BS;
      return 2;
    }
    if (Source[1] == TMsg_BS_EOF) {
      *Dest=TMsg_EOF;
      return 2;
    }
    return 0; // invalide sequence
  } else {
    *Dest=Source[0];
    return 1;
  }  
}

int ReverseByteStuffCopy(TMsg *Dest, u8 *Source)
{
  int Count=0, State=0;

  while ((*Source)!=TMsg_EOF) {
    if (State==0) {
      if ((*Source)==TMsg_BS) {
        State=1;
      } else {
        Dest->Data[Count]=*Source;
        Count++;
      }
    } else {
      if ((*Source)==TMsg_BS) {
        Dest->Data[Count]=TMsg_BS;
        Count++;
      } else {
        if ((*Source)==TMsg_BS_EOF) {
          Dest->Data[Count]=TMsg_EOF;
          Count++;
        } else {
          return 0; // invalid sequence
        }
      }
      State=0;
    }
    Source++;
  }
  if (State!=0) return 0;
  Dest->Len=Count;
  return 1;
}

void CHK_ComputeAndAdd(TMsg *Msg)
{
  u8 CHK=0;
  int i;
  
  for(i=0; i<Msg->Len; i++) {
    CHK-=Msg->Data[i];
  }
  Msg->Data[i]=CHK;
  Msg->Len++;
}

int CHK_CheckAndRemove(TMsg *Msg)
{
  u8 CHK=0;
  int i;
  
  for(i=0; i<Msg->Len; i++) {
    CHK+=Msg->Data[i];
  }
  Msg->Len--;
  return (CHK==0);
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
